#!/usr/bin/env python3
# www.jrodal.com


from dataclasses import dataclass
import re
import textwrap
from typing import List

import openai

from ai_interface import AiInterface


@dataclass
class TextSummary:
    final_summary: str
    chunk_summaries: List[str]


class TextSummarizer(AiInterface):
    def __init__(
        self,
        *,
        model: str = "text-davinci-003",
        prompt_string: str = "Write a detailed summary of the following:\n\n{}\n"
    ) -> None:
        self.model = model
        self.prompt_string = prompt_string

    def _summarize_chunk(self, chunk: str, max_tokens: int = 256) -> str:
        prompt = self.prompt_string.format(chunk)

        response = openai.Completion.create(  # type: ignore
            model="text-davinci-003", prompt=prompt, max_tokens=max_tokens
        )
        summary = re.sub(r"\s+", " ", response.choices[0].text.strip())  # type: ignore
        return summary

    def summarize_text(self, text: str) -> TextSummary:
        text_length = len(text)

        # This is done to reduce overall amount of API calls
        chunk_size = 4000 if text_length >= 25000 else 2000

        # Wrap the transcript in chunks of characters
        chunks = textwrap.wrap(text, chunk_size)

        summaries = [self._summarize_chunk(chunk) for chunk in chunks]
        final_summary = self._summarize_chunk(" ".join(summaries), max_tokens=2056)

        return TextSummary(final_summary=final_summary, chunk_summaries=summaries)



#!/usr/bin/env python3
# www.jrodal.com


class TldrwlException(Exception):
    def __init__(
        self,
        *,
        msg: str,
        cause: str,
        remediation: str,
    ) -> None:
        super().__init__(
            "\n".join(
                (
                    msg,
                    f"Cause: {cause}",
                    f"Remediation: {remediation}",
                ),
            )
        )


class TldrwlRegisterException(TldrwlException):
    @classmethod
    def make_error(cls, *, field: str, env_var: str) -> "TldrwlException":
        return cls(
            msg=f"Failed to register {field}",
            cause=f"Environment variable {env_var} is not set.",
            remediation="Set {env_var} before running script.",
        )



#!/usr/bin/env python3
# www.jrodal.com


from register import Register


class AiInterface:
    def __init_subclass__(cls) -> None:
        if not Register.is_registered():
            Register.register()



import setuptools

with open("README.md") as f:
    long_description = f.read()

with open("requirements.txt") as f:
    install_requires = f.read().splitlines()

setuptools.setup(
    name="tldrwl",
    version="0.0.2",
    author="Jacob Rodal",
    author_email="dev@jrodal.com",
    description="Too long, didn't read/watch/listen",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jrodal98/tldrwl",
    packages=setuptools.find_packages(),
    python_requires=">=3.8",
    include_package_data=True,
    install_requires=install_requires,
)



#!/usr/bin/env python3
# www.jrodal.com


import os
import openai

from exception import TldrwlRegisterException


class Register:
    _registered: bool = False

    @classmethod
    def is_registered(cls) -> bool:
        return cls._registered

    @classmethod
    def register(
        cls,
        *,
        openai_api_key_env_var: str = "OPENAI_API_KEY",
        openai_organization_env_var: str = "OPENAI_ORGANIZATION"
    ) -> None:
        openai_api_key = os.getenv(openai_api_key_env_var)
        if not openai_api_key:
            raise TldrwlRegisterException.make_error(
                field="openai api key", env_var=openai_api_key_env_var
            )
        openai.api_key = openai_api_key

        openai_organization = os.getenv(openai_organization_env_var)
        if not openai_organization:
            raise TldrwlRegisterException.make_error(
                field="openai organization", env_var=openai_organization_env_var
            )
        openai.organization = openai_organization

        cls._registered = True
